@@ -20,7 +20,7 @@ from dotdrop.comparator import Comparator
 from dotdrop.utils import get_tmpdir, remove, strip_home, \
     run, uniq_list, patch_ignores, dependencies_met
 from dotdrop.linktypes import LinkTypes
-from dotdrop.exceptions import YamlException
+from dotdrop.exceptions import YamlException, UndefinedException
 
 LOG = Logger()
 TRANS_SUFFIX = 'trans'
@@ -47,7 +47,7 @@ def action_executor(o, actions, defactions, templater, post=False):
                                                                  action))
                 continue
             if o.debug:
-                LOG.dbg('executing def-{}-action {}'.format(s, action))
+                LOG.dbg('executing def-{}-action: {}'.format(s, action))
             ret = action.execute(templater=templater, debug=o.debug)
             if not ret:
                 err = 'def-{}-action \"{}\" failed'.format(s, action.key)
@@ -60,7 +60,7 @@ def action_executor(o, actions, defactions, templater, post=False):
                 LOG.dry('would execute {}-action: {}'.format(s, action))
                 continue
             if o.debug:
-                LOG.dbg('executing {}-action {}'.format(s, action))
+                LOG.dbg('executing {}-action: {}'.format(s, action))
             ret = action.execute(templater=templater, debug=o.debug)
             if not ret:
                 err = '{}-action \"{}\" failed'.format(s, action.key)
@@ -105,7 +105,7 @@ def cmd_install(o):
 
     # execute profile pre-action
     if o.debug:
-        LOG.dbg('execute profile pre actions')
+        LOG.dbg('run {} profile pre actions'.format(len(pro_pre_actions)))
     ret, err = action_executor(o, pro_pre_actions, [], t, post=False)()
     if not ret:
         return False
@@ -125,7 +125,8 @@ def cmd_install(o):
                                            t, post=False)
 
         if o.debug:
-            LOG.dbg('installing {}'.format(dotfile))
+            LOG.dbg('installing dotfile: \"{}\"'.format(dotfile.key))
+            LOG.dbg(dotfile.prt())
         if hasattr(dotfile, 'link') and dotfile.link == LinkTypes.LINK:
             r, err = inst.link(t, dotfile.src, dotfile.dst,
                                actionexec=pre_actions_exec)
@@ -181,11 +182,15 @@ def cmd_install(o):
     # execute profile post-action
     if installed > 0 or o.install_force_action:
         if o.debug:
-            LOG.dbg('execute profile post actions')
+            msg = 'run {} profile post actions'
+            LOG.dbg(msg.format(len(pro_post_actions)))
         ret, err = action_executor(o, pro_post_actions, [], t, post=False)()
         if not ret:
             return False
 
+    if o.debug:
+        LOG.dbg('install done')
+
     if o.install_temporary:
         LOG.log('\ninstalled to tmp \"{}\".'.format(tmpdir))
     LOG.log('\n{} dotfile(s) installed.'.format(installed))
@@ -225,6 +230,7 @@ def cmd_compare(o, tmp):
         newvars = dotfile.get_dotfile_variables()
         t.add_tmp_vars(newvars=newvars)
 
+        # dotfiles does not exist / not installed
         if o.debug:
             LOG.dbg('comparing {}'.format(dotfile))
         src = dotfile.src
@@ -234,9 +240,9 @@ def cmd_compare(o, tmp):
             same = False
             continue
 
+        # apply transformation
         tmpsrc = None
         if dotfile.trans_r:
-            # apply transformation
             if o.debug:
                 LOG.dbg('applying transformation before comparing')
             tmpsrc = apply_trans(o.dotpath, dotfile, t, debug=o.debug)
@@ -256,20 +262,26 @@ def cmd_compare(o, tmp):
                 LOG.dbg('points to itself')
             continue
 
-        # install dotfile to temporary dir
-        ret, insttmp = inst.install_to_temp(t, tmp, src, dotfile.dst)
+        # install dotfile to temporary dir and compare
+        ret, err, insttmp = inst.install_to_temp(t, tmp, src, dotfile.dst)
         if not ret:
             # failed to install to tmp
+            line = '=> compare {}: error'
+            LOG.log(line.format(dotfile.key, err))
+            LOG.err(err)
             same = False
             continue
         ignores = list(set(o.compare_ignore + dotfile.cmpignore))
         ignores = patch_ignores(ignores, dotfile.dst, debug=o.debug)
         diff = comp.compare(insttmp, dotfile.dst, ignore=ignores)
+
+        # clean tmp transformed dotfile if any
         if tmpsrc:
-            # clean tmp transformed dotfile if any
             tmpsrc = os.path.join(o.dotpath, tmpsrc)
             if os.path.exists(tmpsrc):
                 remove(tmpsrc)
+
+        # print diff result
         if diff == '':
             if o.debug:
                 line = '=> compare {}: diffing with \"{}\"'
@@ -628,7 +640,7 @@ def apply_trans(dotpath, dotfile, templater, debug=False):
     new_src = '{}.{}'.format(src, TRANS_SUFFIX)
     trans = dotfile.trans_r
     if debug:
-        LOG.dbg('executing transformation {}'.format(trans))
+        LOG.dbg('executing transformation: {}'.format(trans))
     s = os.path.join(dotpath, src)
     temp = os.path.join(dotpath, new_src)
     if not trans.transform(s, temp, templater=templater, debug=debug):
@@ -650,7 +662,10 @@ def main():
     try:
         o = Options()
     except YamlException as e:
-        LOG.err('config file error: {}'.format(str(e)))
+        LOG.err('config error: {}'.format(str(e)))
+        return False
+    except UndefinedException as e:
+        LOG.err('config error: {}'.format(str(e)))
         return False
 
     if o.debug:
